package br.usp.poli.java.ihc;
/*
Copyright (C) 2008 Fbio Levy Siqueira (fabiolevy@yahoo.com.br)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
import br.usp.poli.java.ICarta;
import br.usp.poli.java.IJogador;

/**
 * Representa a interface bsica para o Jogador.
 * @author FLevy
 */
abstract class JogadorIHC implements IJogador {
	protected int valorPartida = 1;
	protected Mao mao;
	protected String nome;
	protected Mesa mesa;
	
	public JogadorIHC(String nome, boolean estaEmCima, boolean visivel) {
		this.nome = nome;
		this.mao = new Mao(this, estaEmCima, visivel);
		this.mao.iniciar();
	}
	
	public String getNome() {
		return this.nome;
	}
	
	JogadorIHC getOponente() {
		for (JogadorIHC j : this.mesa.jogadores()) {
			if (j != this) return j;
		}
		
		return null;
	}
	
	void avisarAceitouTruco(int novoValor) {
		this.valorPartida = novoValor;
	}
	
	void setMesa(Mesa mesa) {
		this.mesa = mesa;
	}
	
	public final ICarta jogar(ICarta cartaNaMesa) {
		if (!mao.isEnabled()) throw new ExcecaoParada();
		ICarta cartaJogada = jogarCarta(cartaNaMesa);
		
		if (cartaJogada == null) return null;
		
		mesa.jogarCarta(cartaJogada);
		
		return cartaJogada;
	}
	
	protected abstract ICarta jogarCarta(ICarta cartaNaMesa);
	
	Mao getMao() {
		return mao;
	}
	
	void setEnabled(boolean enabled) {
		if (!enabled) mao.setEnabled(false);
	}
	
	protected void receberCartas(ICarta[] cartas) {
		if (!mao.isEnabled()) throw new ExcecaoParada();
		this.valorPartida = 1;
		this.mao.receberCartas(cartas);
	}
	
	protected void receberCartas(ICarta[] cartas, ICarta vira) {
		if (!mao.isEnabled()) throw new ExcecaoParada();
		this.valorPartida = 1;
		this.mao.receberCartas(cartas);
		
		if (!this.mesa.viraJaDefinido())
			this.mesa.setVira(vira);
	}
	
	public void avisarGanhouMao(boolean ganhou, int valor) {
		if (!mao.isEnabled()) throw new ExcecaoParada();
		if (ganhou) {
			mesa.pontuar(this, valorPartida);
			
			try {
				Thread.sleep(Integer.parseInt(Truco.obterPropriedade("espera_vitoria")));
			} catch(NumberFormatException e) {
				throw new ProblemaInterface("Erro ao obter as propriedades do arquivo de configurao: espera_vitoria no  um nmero.");
			} catch (InterruptedException e) { }
			this.mesa.fimMao();
		}
	}
	
	public void avisarGanhouRodada(boolean ganhou) {
		if (!mao.isEnabled()) throw new ExcecaoParada();
		if (ganhou) {
			try {
				Thread.sleep(Integer.parseInt(Truco.obterPropriedade("espera_comparacao")));
			} catch(NumberFormatException e) {
				throw new ProblemaInterface("Erro ao obter as propriedades do arquivo de configurao: espera_comparacao no  um nmero.");
			} catch (InterruptedException e) { }
			this.mesa.fimRodada(this);
		}
	}
}